<?php
/*
Plugin Name: SETI Stats Display
Plugin URI: http://www.j2fi.net/2007/03/21/setihome-wordpress-plugin/
Description: Display current SETI@Home stats
Version: 0.2
Author: Jason F. Irwin Jfi
Author URI: http://www.j2fi.net/
Disclaimer: Use at your own risk. No warranty expressed or implied is provided.

*/

/*	Copyright 2007  Jason F. Irwin Jfi  (email: jason@j2fi.net)
	
	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/* *****INSTRUCTIONS*****

  Installation
  ============
  Upload the folder "seti-stats" into your "wp-content/plugins"
  Log in to Wordpress Administration area, choose "Plugins" from the main menu, find "SETI Stats", and click the "Activate" button 
  Choose "Options->SETI Options" from the main menu and enter your SETI@Home Account number
  
  Upgrading
  =========
  Just overwrite the previous version and follow the Installation instructions.
  
  Configuration
  =============
  Modify your theme (usually your sidebar.php) by adding the following php code:
  
  get_seti_stats()
  
  Don't forget to wrap this in a proper php structure!
  
  Uninstallation
  ==============
  Log in to Wordpress Administration area, choose Plugins from the main menu, find the name of the plugin SETI Stats, and click the Deactivate button 

  ***********************
  Change Log
  ----------
  0.1 - (Mar 21/07)
	  	- Initial Release
	0.2 - (Mar 22/07)
			- Introduced File Caching
*/

function seti_init() {
  	define('SETI_ACCT', get_option('seti_acct'));
  	define('SETI_EXPY', get_option('seti_expy'));
}
add_action('init','seti_init');

define('SETI_FILE', ABSPATH . 'wp-content/plugins/seti-stats/seti_cache.txt');

function get_seti_stats() {
	//Get the Write Time
	$fh = fopen(SETI_FILE, 'r');
	$expiry = fread($fh, 10);
	fclose($fh);

	//Date Compare
	$today = mktime(date("H"), 0, 0, date("m"), date("d"), date("y"));

	if ($expiry < $today) {
		//Get Fresh Data
		read_seti_site();
	}
	
	//Read Data From Cache
	$fg = fopen(SETI_FILE, 'r');
	$out = fread($fg, filesize(SETI_FILE));
	fclose($fg);

	//Output
	$out = substr($out, 10, strlen($out));
	echo $out;
}

function read_seti_site() {
	$host = 'setiathome.berkeley.edu';
	$path = '/userw.php?id=' . SETI_ACCT;
	$seti_logo = get_settings('home') . '/wp-content/plugins/seti-stats/seti_button.png';
	$seti_url = 'http://setiathome.berkeley.edu';

  $res  = "GET $path HTTP/1.0\r\n";
  $res .= "Host: $host\r\n";
  $res .= "User-Agent: " . $_SERVER['HTTP_USER_AGENT'] . "\r\n";
  $res .= "Content-Type: application/x-www-form-urlencoded\r\n";
  $res .= "Content-Length: 0\r\n";
  $res .= "Connection: close\r\n\r\n";
  
	$fp = @fsockopen($host, 80, $errno, $errstr);
	$buf = '';

	if (!$fp) {
		echo "<li> Error: $errno </li>
				  <li> Descr: $errstr </li>";
		return;
	} else {
		fputs($fp, $res, strlen($res));		//send for data
		while (!feof($fp)) {
			$line = fgets($fp);
			$buf .= $line;
		}		//while
 	}			//if

  fclose($fp);
  
  $credit = strstr($line, 'TotCred: ');
	$avg_cr = strstr($line, 'AvgCred: ');
	$team = strstr($line, 'Team: ');
	
	//Clean Descriptions
	$credit = str_replace('TotCred', 'Credit', $credit);
	$avg_cr = str_replace('AvgCred', 'Average', $avg_cr);

	//Parse Strings
	$credit = substr($credit, 0, strpos($credit, '<'));
	$avg_cr = substr($avg_cr, 0, strpos($avg_cr, '<'));
	$team = substr($team, 0, strpos($team, '<'));

	$expire = mktime(date("H")+SETI_EXPY, 0, 0, date("m"), date("d"), date("y"));

	//Write to Cache
	$fh = fopen(SETI_FILE, 'w') or die("can't open file");
	$stringData = $expire;
	$stringData .= " <li> $credit </li>	<li> $avg_cr </li> <li> $team </li>	<ul> <a href=$seti_url ><img src=$seti_logo /></a> </ul>";
	fwrite($fh, $stringData);
	fclose($fh);
}

/** filters**/
add_action('admin_head', 'seti_add_options_page');

function seti_add_options_page() {
	add_options_page('SETI Options', 'SETI Options', 'manage_options', 'seti-stats/options-seti.php');

	$expire = mktime(date("H")-SETI_EXPY, 0, 0, date("m"), date("d"), date("y"));

	//Create the Cache
	$fh = fopen(SETI_FILE, 'w') or die("can't open file");
	$stringData = $expire;
	fwrite($fh, $stringData);
	fclose($fh);
}

?>